@@ -45,22 +45,33 @@
  * booleans or conditional rules are thrown away a warning is printed.
  */
 
-#include "policydb.h"
-#include "services.h"
-#include "queue.h"
-#include "checkpolicy.h"
-#include "conditional.h"
 #include <getopt.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <errno.h>
 #include <sys/mman.h>
+
+#include <sepol/policydb.h>
+#include <sepol/services.h>
+#include <sepol/conditional.h>
+#include <sepol/flask.h>
+
+#include "queue.h"
+#include "checkpolicy.h"
+
 extern char *optarg;
 extern int optind;
 
+static policydb_t policydb;
+static sidtab_t sidtab;
+
 extern policydb_t *policydbp;
 extern queue_t id_queue;
 extern unsigned int policydb_errors;
@@ -68,9 +79,6 @@ extern unsigned long policydb_lineno;
 extern unsigned long source_lineno;
 extern char source_file[];
 extern unsigned int pass;
-extern unsigned int ss_initialized;
-
-extern int policydb_write(policydb_t * p, FILE * fp);
 
 extern FILE *yyin;
 extern int yyparse(void);
@@ -79,12 +87,7 @@ extern void yyrestart(FILE *);
 char *txtfile = "policy.conf";
 char *binfile = "policy";
 
-int policyvers = POLICYDB_VERSION_MAX;
-
-int avc_ss_reset(__u32 seqno __attribute__ ((unused)))
-{
-	return 0;
-}
+unsigned int policyvers = POLICYDB_VERSION_MAX;
 
 void usage(char *progname)
 {
@@ -100,7 +103,7 @@ static int print_sid(security_id_t sid,
 	size_t scontext_len;
 	int rc;
 
-	rc = security_sid_to_context(sid, &scontext, &scontext_len);
+	rc = sepol_sid_to_context(sid, &scontext, &scontext_len);
 	if (rc)
 		printf("sid %d -> error %d\n", sid, rc);
 	else {
@@ -275,9 +278,9 @@ static void cond_check_type_rules_list(cond_av_list_t *list)
 		if (cur->node->datum.specified & AVTAB_TYPE) {
 			if (avtab_search(&policydbp->te_avtab, &cur->node->key, AVTAB_TYPE)) {
 				fprintf(stderr, "conditional type rule for (%s, %s : %s) conflicts with entry in base policy; "
-					"conditional rule discarded.\n", policydbp->p_type_val_to_name[cur->node->key.source_type],
-					policydbp->p_type_val_to_name[cur->node->key.target_type],
-					policydbp->p_class_val_to_name[cur->node->key.target_class]);
+					"conditional rule discarded.\n", policydbp->p_type_val_to_name[cur->node->key.source_type-1],
+					policydbp->p_type_val_to_name[cur->node->key.target_type-1],
+					policydbp->p_class_val_to_name[cur->node->key.target_class-1]);
 				cur->node->parse_context = (void*)0;
 			} else {
 				cur->node->parse_context = (void*)1;
@@ -302,7 +305,7 @@ static void cond_check_type_rules(void)
 	}
 }
 
-extern char *av_to_string(__u32 tclass, access_vector_t av);
+extern char *av_to_string(uint32_t tclass, access_vector_t av);
 
 void check_assertion_helper(unsigned int stype, unsigned int ttype, ebitmap_t *tclasses, 
 			    access_vector_t *avp, unsigned long line) 
@@ -453,6 +456,7 @@ int main(int argc, char **argv)
 	char *name;
 	int state;
 	int show_version = 0;
+	struct policy_file pf;
 
 
 	while ((ch = getopt(argc, argv, "o:dbVc:")) != EOF) {
@@ -509,6 +513,12 @@ int main(int argc, char **argv)
 	printf("%s:  loading policy configuration from %s\n", argv[0],
 	       file);
 
+	/* Set policydb and sidtab used by libsepol service functions
+	   to my structures, so that I can directly populate and
+	   manipulate them. */
+	sepol_set_policydb(&policydb);
+	sepol_set_sidtab(&sidtab);
+
 	if (binary) {
 		fd = open(file, O_RDONLY);
 		if (fd < 0) {
@@ -527,7 +537,10 @@ int main(int argc, char **argv)
 				file, strerror(errno));
 			exit(1);
 		}
-		ret = security_load_policy(map,sb.st_size);
+		pf.type = PF_USE_MEMORY;
+		pf.data = map;
+		pf.len = sb.st_size;
+		ret = policydb_read(&policydb, &pf, 1);
 		if (ret) {
 			fprintf(stderr, "%s:  error(s) encountered while parsing configuration\n", argv[0]);
 			exit(1);
@@ -575,6 +588,12 @@ int main(int argc, char **argv)
 		if (policydb_errors) 
 			exit(1);
 
+		if (policyvers >= POLICYDB_VERSION_NLCLASS &&
+		    policydb.p_classes.nprim < SECCLASS_NETLINK_DNRT_SOCKET) {
+			fprintf(stderr, "%s:  policy lacks new netlink classes, unable to generate policy version %d\n", argv[0], policyvers);
+			exit(1);
+		}
+	    
 		/* remove type attributes */
 		hashtab_map_remove_on_error(policydb.p_types.table, 
 					    type_attr_remove, 0, 0);
@@ -594,7 +613,12 @@ int main(int argc, char **argv)
 			perror(outfile);
 			exit(1);
 		}
-		ret = policydb_write(&policydb, outfp);
+
+		sepol_set_policyvers(policyvers);
+
+		pf.type = PF_USE_STDIO;
+		pf.fp = outfp;
+		ret = policydb_write(&policydb, &pf);
 		if (ret) {
 			fprintf(stderr, "%s:  error writing %s\n",
 				argv[0], outfile);
@@ -605,8 +629,6 @@ int main(int argc, char **argv)
 	if (!debug)
 		exit(0);
 
-	ss_initialized = 1;
-
       menu:
 	printf("\nSelect an option:\n");
 	printf("0)  Call compute_access_vector\n");
@@ -669,7 +691,7 @@ int main(int argc, char **argv)
 				printf("\nNo access vector definition for that class\n");
 				break;
 			}
-			ret = security_compute_av(ssid, tsid, tclass, 0,
+			ret = sepol_compute_av(ssid, tsid, tclass, 0,
 						  &avd);
 			switch (ret) {
 			case 0:
@@ -700,7 +722,7 @@ int main(int argc, char **argv)
 			printf("sid?  ");
 			fgets(ans, sizeof(ans), stdin);
 			ssid = atoi(ans);
-			ret = security_sid_to_context(ssid,
+			ret = sepol_sid_to_context(ssid,
 					       &scontext, &scontext_len);
 			switch (ret) {
 			case 0:
@@ -722,7 +744,7 @@ int main(int argc, char **argv)
 			fgets(ans, sizeof(ans), stdin);
 			scontext_len = strlen(ans);
 			ans[scontext_len - 1] = 0;
-			ret = security_context_to_sid(ans, scontext_len,
+			ret = sepol_context_to_sid(ans, scontext_len,
 						      &ssid);
 			switch (ret) {
 			case 0:
@@ -770,11 +792,11 @@ int main(int argc, char **argv)
 			}
 
 			if (ch == '3')
-				ret = security_transition_sid(ssid, tsid, tclass, &ssid);
+				ret = sepol_transition_sid(ssid, tsid, tclass, &ssid);
 			else if (ch == '4')
-				ret = security_member_sid(ssid, tsid, tclass, &ssid);
+				ret = sepol_member_sid(ssid, tsid, tclass, &ssid);
 			else 
-				ret = security_change_sid(ssid, tsid, tclass, &ssid);
+				ret = sepol_change_sid(ssid, tsid, tclass, &ssid);
 			switch (ret) {
 			case 0:
 				printf("\nsid %d\n", ssid);
@@ -790,7 +812,7 @@ int main(int argc, char **argv)
 			}
 			break;
 		case '6':
-			sidtab_map(&sidtab, print_sid, 0);
+			sepol_sidtab_map(&sidtab, print_sid, 0);
 			break;
 		case '7':
 			printf("pathname?  ");
@@ -809,13 +831,13 @@ int main(int argc, char **argv)
 					ans, strerror(errno));
 				break;
 			}
-			map = mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, fd, 0);
+			map = mmap(NULL, sb.st_size, PROT_READ | PROT_WRITE, MAP_PRIVATE, fd, 0);
 			if (map == MAP_FAILED) {
 				fprintf(stderr, "Can't map '%s':  %s\n",
 					ans, strerror(errno));
 				break;
 			}
-			ret = security_load_policy(map,sb.st_size);
+			ret = sepol_load_policy(map,sb.st_size);
 			switch (ret) {
 			case 0:
 				printf("\nsuccess\n");
@@ -834,7 +856,7 @@ int main(int argc, char **argv)
 			printf("fs kdevname?  ");
 			fgets(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
-			security_fs_sid(ans, &ssid, &tsid);
+			sepol_fs_sid(ans, &ssid, &tsid);
 			printf("fs_sid %d default_file_sid %d\n",
 			       ssid, tsid);
 			break;
@@ -853,14 +875,14 @@ int main(int argc, char **argv)
 			printf("port? ");
 			fgets(ans, sizeof(ans), stdin);
 			port = atoi(ans);
-			security_port_sid(0, 0, protocol, port, &ssid);
+			sepol_port_sid(0, 0, protocol, port, &ssid);
 			printf("sid %d\n", ssid);
 			break;
 		case 'a':
 			printf("netif name?  ");
 			fgets(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
-			security_netif_sid(ans, &ssid, &tsid);
+			sepol_netif_sid(ans, &ssid, &tsid);
 			printf("if_sid %d default_msg_sid %d\n",
 			       ssid, tsid);
 			break;
@@ -899,7 +921,7 @@ int main(int argc, char **argv)
 				break;
 			}
 
-			security_node_sid(family, p, len, &ssid);
+			sepol_node_sid(family, p, len, &ssid);
 			printf("sid %d\n", ssid);
 			break;
 		}	
@@ -907,7 +929,7 @@ int main(int argc, char **argv)
 			printf("fstype?  ");
 			fgets(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
-			security_fs_use(ans, &ret, &ssid);
+			sepol_fs_use(ans, &ret, &ssid);
 			switch (ret) {
 			case SECURITY_FS_USE_XATTR:
 				printf("use xattr\n");
@@ -954,7 +976,7 @@ int main(int argc, char **argv)
 				}
 				tclass = cladatum->value;
 			}
-			security_genfs_sid(fstype, path, tclass, &ssid);
+			sepol_genfs_sid(fstype, path, tclass, &ssid);
 			printf("sid %d\n", ssid);
 			free(fstype);
 			free(path);
@@ -969,7 +991,7 @@ int main(int argc, char **argv)
 			fgets(ans, sizeof(ans), stdin);
 			ans[strlen(ans) - 1] = 0;
 
-			ret = security_get_user_sids(ssid, ans, &sids, &nel);
+			ret = sepol_get_user_sids(ssid, ans, &sids, &nel);
 			switch (ret) {
 			case 0:
 			        if (!nel)
